%  The DCG here is taken from the ACL86 paper.

module acldcg.

import kltypes.


% Declare the types for the grammar's categories.
type sentence     bool ->                       (list string) -> (list string) -> o.
type np           ((i -> bool) -> bool) ->      (list string) -> (list string) -> o.
type vp           (i -> bool) ->                (list string) -> (list string) -> o.
type relcl        ((i -> bool) -> i -> bool) -> (list string) -> (list string) -> o.
type determ       ((i -> bool) -> (i -> bool) -> bool) -> 
                                                (list string) -> (list string) -> o.
type noun         (i -> bool) ->                (list string) -> (list string) -> o.
type propernoun   i ->                          (list string) -> (list string) -> o.
type transverb    (i -> i -> bool) ->           (list string) -> (list string) -> o.
type intransverb  (i -> bool) ->                (list string) -> (list string) -> o.
type fullstop                                   (list string) -> (list string) -> o.

sentence (P1 P2) L1 L4 :- np P1 L1 L2, vp P2 L2 L3, fullstop L3 L4.
np (P1 (P3 P2)) L1 L4  :- determ P1 L1 L2, noun P2 L2 L3, relcl P3 L3 L4.
np P\(P X) L1 L2       :- propernoun X L1 L2.
vp X\(P2 (P1 X)) L1 L3 :- transverb P1 L1 L2, np P2 L2 L3.
vp P L1 L2             :- intransverb P L1 L2.
relcl P1\X\((P1 X) & (P2 X)) [that|L1] L2 :-  vp P2 L1 L2.
relcl P\X\(P X) L1 L1.
determ P1\P2\(P2 (iota P1)) ['the'|L1] L1.
determ P1\P2\(all X\((P1 X) ==> (P2 X))) ['every'|L1] L1.
determ P1\P2\(some X\((P1 X) & (P2 X))) ['a'|L1] L1.

noun		man	['man'|L1] L1.
noun 		woman	['woman'|L1] L1.
propernoun	john	['john'|L1] L1.
propernoun	mary	['mary'|L1] L1.
transverb	loves	['loves'|L1] L1.
transverb	likes	['likes'|L1] L1.
intransverb	lives	['lives'|L1] L1.
fullstop 		['fs'|L1] L1.

