module ndtac.

%  Below are the definition of the primitive tactics for a fragment
%  of first-order logic.  The proof objects which are built are in 
%  the form of natural deduction proofs.

import  goaltypes ndproofs listmanip.

infix   130 :: xfx.
infix   140 --> xfx.

kind    judgment	type.
kind    answer		type.

type    ::		proof_object -> bool -> judgment.
type    -->		(list judgment) -> judgment -> goal.

type	yes		answer.

type	exists_e_tac	int -> goal -> goal -> o.
type	or_e_tac	int -> goal -> goal -> o.
type	forall_e_query	int -> goal -> goal -> o.
type	forall_e_tac	int -> goal -> goal -> o.
type	fchain_tac	int -> goal -> goal -> o.
type	bchain_tac	int -> goal -> goal -> o.
type	imp_e_retain	int -> goal -> goal -> o.
type	imp_e_tac	int -> goal -> goal -> o.
type	and_e_tac	int -> goal -> goal -> o.
type	exists_i_query	goal -> goal -> o.
type	exists_i_tac	goal -> goal -> o.
type	forall_i_tac	goal -> goal -> o.
type	imp_i_tac	goal -> goal -> o.
type	or_i2_tac	goal -> goal -> o.
type	or_i1_tac	goal -> goal -> o.
type	and_i_tac	goal -> goal -> o.
type	close_tacn	int -> goal -> goal -> o.
type	close_tac	goal -> goal -> o.


close_tac (Gamma --> P :: A) truegoal :-
  member (P :: A) Gamma.

close_tacn N (Gamma --> P :: A) truegoal :-
  nth_item N (P :: A) Gamma.

and_i_tac (Gamma --> (and_i P1 P2) :: A and B)
          (andgoal (Gamma --> P1 :: A) (Gamma --> P2 :: B)).

or_i1_tac (Gamma --> (or_i1 P) :: A or B)
          (Gamma --> P :: A).

or_i2_tac (Gamma --> (or_i2 P) :: A or B)
          (Gamma --> P :: B).

imp_i_tac (Gamma --> (imp_i P) :: A imp B)
          (allgoal PA\ ([(PA :: A)| Gamma] --> (P PA) :: B)).

forall_i_tac (Gamma --> (forall_i P) :: forall A)
             (allgoal T\ (Gamma --> (P T) :: (A T))).

exists_i_tac (Gamma --> (exists_i T P) :: exists A)
             (Gamma --> P :: (A T)).

exists_i_query (Gamma --> (exists_i T P) :: exists A)
               (Gamma --> P :: (A T)) :-
  writesans 'Enter substitution term: ', read T.

and_e_tac N (Gamma --> PC :: C)
            ([(and_e1 P) :: A, (and_e2 P) :: B | Gamma1] --> PC :: C) :-
  nth_item_and_rest N (P :: A and B) Gamma Gamma1.

imp_e_tac N (Gamma --> PC :: C)
            (andgoal (Gamma1 --> PA :: A)
                     ([(imp_e PA P) :: B | Gamma1] --> PC :: C)) :-
  nth_item_and_rest N (P :: A imp B) Gamma Gamma1.

imp_e_retain N (Gamma --> PC :: C)
               (andgoal (Gamma --> PA :: A)
                        ([(imp_e PA P) :: B | Gamma] --> PC :: C)) :-
  nth_item N (P :: A imp B) Gamma.

bchain_tac N (Gamma --> (imp_e PA P) :: B)
             (Gamma1 --> PA :: A) :-
  nth_item_and_rest N (P :: A imp B) Gamma Gamma1.

fchain_tac N (Gamma --> PC :: C)
             ([(imp_e PA P) :: B | Gamma2] --> PC :: C) :-
  nth_item_and_rest N (P :: A imp B) Gamma Gamma1,
  member_and_rest (PA :: A) Gamma1 Gamma2.

forall_e_tac N (Gamma --> PC :: C)
               ([(forall_e T P) :: (A T) | Gamma1] --> PC :: C) :-
  nth_item_and_rest N (P :: forall A) Gamma Gamma1.

forall_e_query N (Gamma --> PC :: C)
                 ([(forall_e T P) :: (A T) | Gamma1] --> PC :: C) :-
  writesans 'Enter substitution term: ', read T,
  writesans 'Remove hypothesis? ',
  (read yes, nth_item_and_rest N (P :: forall A) Gamma Gamma1;
   Gamma1 = Gamma, nth_item N (P :: forall A) Gamma).

or_e_tac N (Gamma --> (or_e P P1 P2) :: C)
          (andgoal (allgoal PA\ ([PA :: A | Gamma1] --> (P1 PA) :: C))
                   (allgoal PB\ ([PB :: B | Gamma1] --> (P2 PB) :: C))) :-
  nth_item_and_rest N (P :: A or B) Gamma Gamma1.

exists_e_tac N (Gamma --> (exists_e P PC) :: C)
               (allgoal T\ (allgoal PA\
                 ([PA :: (A T) | Gamma1] --> (PC T PA) :: C))) :-
  nth_item_and_rest N (P :: exists A) Gamma Gamma1.
