%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%	CAM : Categorical Abstract Machine		  %%%
%%%	Declaration of Command and Stack Constructors	  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module camdecl.

infix	40	&	yfx.		%%% composition (sequence)

%%% Command and Stack Kinds %%%
kind	com	type.			%%% CAM commands
kind	val	type.			%%% type of elements on stack (values)
kind	prog	type.			%%% program
kind	stk	type.			%%% stack
kind	oper	type.			%%% primitive operators

%%% Command constructors for CAM %%%
type	quote	val -> com.
type	skip	com.
type	cfst	com.
type	csnd	com.
type	cons	com.
type	push	com.
type	swap	com.
type	op	oper -> com.		%%% coerce primitives to commands
type	&	com -> com -> com.	%%% sequence
type	branch	com -> com -> com.
type	cur	com -> com.
type	app	com.
type	rec	com -> com.
type	program com -> prog.		%%% program constructor

%%% Stack constructor
type	stk	(list val) -> stk.	%%% a stack is a list of values

%%% Value constructors
type	ctt	val.			%%% true
type	cff	val.			%%% false
type	##	int -> val.		%%% coerce integers to values
type	pr	val -> val -> val.	%%% pairing (for CAM environments)
type	clist	val -> val -> val.	%%% internal lists
type	cnil	val.			%%% the empty list
type	closure	com -> val -> val.	%%% closure
type	cfix	(val -> val) -> val.	%%% recursive closure
%type	empty	val.			%%% empty environment

%%% Builtin Operators
type	chd	oper.			%%% head of list
type	ctl	oper.			%%% tail of list
type	cempty	oper.			%%% empty list
type	ccons	oper.			%%% cons list
type	cequal	oper.			%%% =
type	cnequal	oper.			%%% not =
type	cplus	oper.			%%% integer +
type	cminus	oper.			%%% integer -
type	ctimes	oper.			%%% integer *

%%%   End of module camdecl   %%%
