%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%        mini-ML: Examples                              %%%
%%%        Examples for testing type inferencing          %%%
%%%            and evaluator.                             %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

module mltest.

import mltype mleval.

%%% FUNCTION DEFINITIONS %%%

%%% factorial function
fact1 (fix Fact\ (lamb N\ (cond (== @ N @ (# 0))
                                (# 1)
                                (times @ N @ (Fact @ (minus @ N @ (# 1))))))).

%%% append function
app1 (fix App\ (lamb K\ (lamb L\
                (cond (== @ K @ nil) 
                      L 
                      (:: @ (hd @ K) @ (App @ (tl @ K) @ L)))))).

%%% mutual recursion example: even/odd functions
evenodd1 (fix EO\ 
           ((lamb X\ (cond (== @ X @ (# 0)) 
                           tt 
                           ((snd @ EO) @ (minus @ X @ (# 1))))) &&
            (lamb X\ (cond (== @ X @ (# 0)) 
                           ff
                           ((fst @ EO) @ (minus @ X @ (# 1))))))).

%%% TEST CASES %%%
test1 T :- fact1 F, infer [] F T.
test2 T :- fact1 F, infer [] (let Fact\(Fact @ (# 3)) F) T.
test3 V :- fact1 F, eval  (let Fact\(Fact @ (# 3)) F) V.
test4 T :- app1 A, infer [] A T.
test5 T :- app1 A, infer [] (let App\(App @ (:: @ (# 1) @ (:: @ (# 2) @ nil))
                                       @ (:: @ (# 3) @ (:: @ (# 4) @ nil))) A)
                         T.
test6 V :- app1 A, eval  (let App\(App @ (:: @ (# 1) @ (:: @ (# 2) @ nil))
                                       @ (:: @ (# 3) @ (:: @ (# 4) @ nil))) A)
                         V.
test7 T :- evenodd1 F, infer [] F T.
test8 V :- evenodd1 F, eval (let EO\((fst @ EO) @ (# 3)) F) V.

%%%   End of module mltest   %%%

