
%  This file contains simple examples of how the functional
%  programming language can be written.  If this module and streams
%  are both imported into the main, then all of these functions are
%  available within calls to the is predicate.

module exps.

import streams.

type	zeros		stream int.
type	successor	int -> int.
type	ints		stream int.
type	initial_ints	int -> stream int.
type	sum		int -> int.
type	fact		int -> int.
type	fib		int -> int.
type	map		(A1 -> A2) -> list A1 -> list A2.
type	app		list A1 -> list A1 -> list A1.
type	mem		A1 -> list A1 -> bool.
type	bounded_list	int -> int -> list int.
type	sum_up		int -> int -> int.


defun zeros
  (fixpt Z\ (scons 0 Z)).

defun successor
  (X\ (X + 1)).

defun ints
  (fixpt Z\ (scons 0 (map_stream successor Z))).

defun initial_ints
   Init\ (fixpt Z\ (scons Init (map_stream successor Z))).

defun sum 
  N\ (iterate (X\Y\ (X + Y)) 0 (chop (X\ (X < N)) ints)).

defun fact
  (fixpt Fact\N\ (if (zerop N) 1 (N * (Fact (N - 1))))).

defun fib
  (fixpt Fib\N\ (if (zerop N) 0
		  (if (equal N 1) 1
		    ((Fib (N - 1)) + (Fib (N - 2)))))).

defun map 
  (fixpt (Map\F\L\ (if (null L) [] 
		     [(F (car L))|(Map F (cdr L))]))).

defun app 
  (fixpt App\L\K\ (if (null L) K
                     (if (consp L) [(car L)|(App (cdr L) K)] []))).

defun mem 
  (fixpt Mem\X\L\ (if (null L) false
		    (if (and (consp L) (equal (car L) X))
			truth
		      (Mem X (cdr L))))).
defun bounded_list
  Lower\Upper\ (chop (W\ (Upper > W)) (initial_ints Lower)).

defun sum_up 
  Lower\Upper\ (iterate (X\Y\ (X + Y)) 0 (bounded_list Lower Upper)).
