%  This module implements a simple interactive resolution theorem
%  prover.  It demonstrates the nature of universal quantification and
%  the inability of LProlog to compute mgu explicitly.

module itp.

import	classical lists.

kind	token	type.

type	yes	token.
type	no	token.
type	pi_list	(i -> (list bool)) -> (list bool).
type	select	int -> (list A) -> A -> (list A) -> o.
type	resolve	(list bool) -> int -> (list bool) -> int -> (list bool) -> o.
type	itp	(list (list bool)) -> (list (list bool)) -> o.

select 1 [X|L] X L :- !.
select N [X|L] Y [X|K] :- N1 is (N - 1), select N1 L Y K.

complementary (neg L) L.
complementary L (neg L).

resolve (pi_list Clause1) L1 Clause2 L2 (pi_list NewClause) :- !,
  (pi X\ (sigma T\ (resolve (Clause1 T) L1 Clause2 L2 (NewClause X)))).
resolve Clause1 L1 (pi_list Clause2) L2 (pi_list NewClause) :- !,
  (pi X\ (sigma T\ (resolve Clause1 L1 (Clause2 T) L2 (NewClause X)))).
resolve Clause1 L1 Clause2 L2 NewClause :-
  select L1 Clause1 Lit1 Rest1,
  select L2 Clause2 Lit2 Rest2,
   writesans 'Resolving the following two literals:',nl,
   write Lit1,nl,
   write Lit2,nl,
  complementary Lit1 Lit2,
  append Rest1 Rest2 NewClause.

display_clauses [] :- !.
display_clauses [C|Cs] :- write C, nl, display_clauses Cs.

itp Clauses_in Clauses_out :-
  nl,writesans 'Current clauses are:',nl,
  display_clauses Clauses_in,
  writesans 'more',
  (read no, !, Clauses_in = Clauses_out; 
   read C1, read L1, read C2, read L2,
   select C1 Clauses_in C Rest1,
   select C2 Clauses_in D Rest2,
   resolve C L1 D L2 E,
   itp [E|Clauses_in] Clauses_out).
itp Clauses Clauses.

clauses1 [(pi_list X\ [(neg (q X)), (neg (a X)),(p X)]),
          (pi_list X\ [(q (f X))]),
                      [(a (f b))],
                      [(a (f c))]].

test B :- clauses1 A, itp A B.

hope C :-
  resolve (pi_list X\ [(neg (q X)), (neg (a X)), (p X)])
          1
	  (pi_list X\ [(q (f X))])
	  1
          C.

