%  This file contains declarations defining the constants used in the
%  various program transformation examples in the slp87 paper.  Notice
%  that several of these declarations are already made in some of the
%  system modules.  This should not proof to be a problem.


module		constants.

infix	25	==	xfx.
infix	25	<	xfx.
infix	25	-	xfx.
infix	25	*	yfx.
infix	25	+	yfx.
infix	45	&	xfy.
infix	50	:=	xfx.
infix	60	-->	xfx.


% The domain type for our simple languages
kind	val	type.

% These types are used to make distinctions in the types of objects in our 
% simple program constructs. The slp paper assumes all these are actually val.
kind	list    type -> type.
kind	pair    type -> type -> type.
kind	bool    type.

% The type of ``function definitions''
kind	assignment	type.

% These types are needed for iterative programs in the tail rec example
kind	cmd	type.           % type of (iterative) commands
kind	reg     type.           % type of locations in store



%  The stuff recursion is made of.
type	cond	bool -> A -> A -> A.
type	fixpt	(A -> A) -> A.

% Boolean operations; To be consistent with slp paper, bool to become val
type	false	bool.
type	truth	bool.
type	and	bool -> bool -> bool.
type	or	bool -> bool -> bool.
type	nt	bool -> bool.


% Integer operations; the int and bool here should really be replaced by val
type	<	int -> int -> bool.
type	==	int -> int -> bool.  %  = is reserved for LP.
type	-	int -> int -> int.
type	*	int -> int -> int.
type	+	int -> int -> int.

%  Types for the pairing datatype; (pair A B) should really become val
type	pair	A -> B -> (pair A B).
type	fst	(pair A B) -> A.
type	snd	(pair A B) -> B.
type	pairp	(pair A B) -> bool.

%  Types for the list datatype; (list A) should really be val
type	cons	A -> (list A) -> (list A).
type	cdr	(list A) -> (list A).
type	car	(list A) -> A.
type	null	(list A) -> bool.
type	nil	(list A).

%  For encoding iterative programs; e.g. assignments and while-loops.
%  Here again, to be consistent with the slp paper, bool --> val
type	if	bool -> cmd -> cmd -> cmd. 
type	while	bool -> cmd -> cmd.        
type	&	cmd -> cmd -> cmd.
type	:=	reg -> A -> cmd.
type	find	reg -> A.

% These are some locations for iterative programs
type	done	reg.
%type	output	reg
%type	loc1	reg
%type	loc2	reg
%type	temp	reg

%  For encoding programs containing ``patterns''
type	-->	A -> A -> assignment.
type	local	(A -> assignment) -> assignment.
type	usepat	(list assignment) -> A.

