%  This file contains a simple "functional-style" interpreter for a
%  Pascal-like language.  This was not discussed in SLP87.

module pas_interp.
import constants.

kind lst type.
kind pair type -> type -> type.

type hnil lst.
type hcons A -> lst -> lst.
type pair  A -> B -> (pair A B).

type test cmd -> A -> B -> lst -> o.
type intp cmd -> lst -> lst -> o.
type express A -> A -> lst -> o.

intp (while Cond Body) In_alist Out_alist :-
   express Cond Boolean In_alist,
   (Boolean = truth, intp (Body & (while Cond Body)) In_alist Out_alist;
    In_alist = Out_alist).

intp (Body1 & Body2) In_alist Out_alist :-
   intp Body1 In_alist Tmp_alist,
   intp Body2 Tmp_alist Out_alist.

intp (if Cond Branch1 Branch2) In_alist Out_alist :-
   express Cond Boolean In_alist,
   (Boolean = truth, intp Branch1 In_alist Out_alist;
    intp Branch2 In_alist Out_alist).

intp (Loc := Exp) In_alist (hcons (pair Loc Val) In_alist) :-
  express Exp Val In_alist.

express (find Loc) Val (hcons (pair Loc Val) Alist) :- !.
express (find Loc) Val (hcons Y Alist) :- express (find Loc) Val  Alist.
express (Exp1 + Exp2) Val Alist :- 
  express Exp1 Val1 Alist,
  express Exp2 Val2 Alist,
  Val is Val1 + Val2.
express (Exp1 - Exp2) Val Alist :- 
  express Exp1 Val1 Alist,
  express Exp2 Val2 Alist,
  Val is Val1 - Val2.
express (Exp1 * Exp2) Val Alist :- 
  express Exp1 Val1 Alist,
  express Exp2 Val2 Alist,
  Val is Val1 * Val2.
express (Exp1 == Exp2) Val Alist :-
  express Exp1 Val1 Alist,
  express Exp2 Val2 Alist,
  ( Val1 = Val2, Val = truth; Val = false).
express (Exp1 < Exp2) Val Alist :-
  express Exp1 Val1 Alist,
  express Exp2 Val2 Alist,
  ( truth is (Val1 < Val2), Val = truth; Val = false).
express (cons Exp1 Exp2) (cns Val1 Val2) Alist :-
  express Exp1 Val1 Alist,
  express Exp2 Val2 Alist.
express (car Exp) Val Alist :-
  express Exp (cons Val Val0) Alist.
express (cdr Exp) Val Alist :-
  express Exp (cons Val0 Val) Alist.
express (null Exp) Val Alist :-
  express Exp Val0 Alist,
  (Val0 = nil, Val = truth; Val = false).

express Val Val Alist.

test Pas X Y Alist :- 
   intp Pas (hcons (pair not_done truth) 
            (hcons (pair loc1 X)
            (hcons (pair loc2 Y) hnil))) Alist.

