% The clauses in this file implement the tail recursion removal talked 
% of in the slp87 paper.

module rem_tailrec.

import constants example_tr.

% predicate that transforms tail recursion (in two place functions) into 
% iteration. The main work is done by trans_body. trans_tailrec merely
% removes prepares the input and the output.
trans_tailrec (fixpt Prog)
              (while (nt (find done)) G) :- 
					trans_body Prog G.


% trans_body recognises a class of terms that correspond to the bodies
% of tail-recursive programs, and transforms these into the bodies of 
% corresponding while loops. The transformation is recursive on the structure
% of the input `tail-recursive' term.

% This is the case when there is no recursive call. This generates code to 
% terminate the iteration.
trans_body (F\X\Y\ (H X Y))
           ((output := (H (find loc1) (find loc2))) &
           (done := truth)).

% The other base case is when there is only a recursive call. In this case the 
% variables need to be reset in iteration. There are some special cases when
% a temporary location is not needed in resetting registers
trans_body (F\X\Y\ (F (G X Y) Y))
           (loc1 :=  (G (find loc1) (find loc2))).
trans_body (F\X\Y\ (F (G X Y) (H Y)))
           ((loc1 := (G (find loc1) (find loc2))) &
           (loc2 := (H (find loc2)))).
trans_body (F\X\Y\ (F (G X Y) (H X Y)))
           ((temp := (find loc1)) &
           (loc1 := (G (find temp) (find loc2))) &
           (loc2 := (H (find temp) (find loc2)))).

% This is the recursive case. If the recursive program has a top level 
% conditional where the left and right arms are tail-recursive, introduce
% a conditional in iteration using code generated for the arms.
trans_body (F\X\Y\ (cond (C X Y) (H1 F X Y) (H2 F X Y)))
           (if (C (find loc1) (find loc2)) G1 G2) :-
				trans_body H1 G1, trans_body H2 G2.


% For testing the above predicates
rem_tailrec Name Pas :- prog Name Fp, trans_tailrec Fp Pas.


