
% These clauses do some equational reasoning about constructors and 
% destructors that is needed for the ``patterns'' example

module solve_pat.

import	constants.

type	force_pat		(A -> A) -> (A -> bool) -> o.
type	intersect_regions	(A -> bool) -> (A -> bool) -> (A -> bool) -> o.

% Given predicates defining two regions, find a predicate that defines
% the intersection of these two regions.
intersect_regions (W\ truth) R R :- !.  % Intersection with ``universal set''.
intersect_regions R (W\ truth) R :- !.  % Same as above.
intersect_regions R1 R2 (W\ (and (R1 W) (R2 W))).

% Given a function term, find the region over which it is a `semantic' identity

% Z\Z is an identity function regardless of the given object
force_pat (Z\ Z) (W\ truth) :- !.  

% Z\nil is an identity function for all objects of which null is true
force_pat (Z\ nil) null :- !.

% (Z \ (pair (F (fst Z)) (G (snd Z)))) is an identity function over pairs if
% F is an id over the first component of the pair and G over the second
force_pat (Z\ (pair (F (fst Z)) (G (snd Z)))) R :-
	force_pat F R1, force_pat G R2,
	intersect_regions (W\ (R1 (fst W))) (W\ (R2 (snd W))) R3,
	intersect_regions pairp R3 R.

% (Z\ (cons (F (car Z)) (G (cdr Z)))) is an id over lists if F is an id over
% the first element of the list and G over the tail of the list
force_pat (Z\ (cons (F (car Z)) (G (cdr Z)))) R :-
	force_pat F R1, force_pat G R2,
	intersect_regions (W\ (R1 (car W))) (W\ (R2 (cdr W))) R3,
	intersect_regions consp R3 R.

