% This is the type inference for an ML-like
% core language with a 'let' that is generically typed.
% It extends the 'termtype' and 'vartype' predicates from tp_simple.

module tp_ml.

import tp_simple tp_types.

type  tlet  (term -> term) -> term -> term.   % generically typed let.
type  gvarbinding  term -> term -> o.

% gvarbinding X V0
% remembers that X is a generically typed variable with "definition"
% V0.  V0 and therefore its free type variables are replicated wherever
% X appears.  Note that  "let f = \x.x in e(x)" is NOT the same as
% "let f = \x:A.x in e(x)", because the free type variable "A" must
% be instantiated consistently.

termtype (tlet U0 V0) (tlet U1 V1) Alpha :- !,
  termtype V0 V1 ArgType,
  (pi X\ (gvarbinding X V0 => termtype (U0 X) (U1 X) Alpha)).

vartype X Alpha :- gvarbinding X V0, !, termtype V0 V1 Alpha.

