% Type Inference for a simply typed lambda calculus.
% Author: Frank Pfenning (fp@cs.cmu.edu).
%
% This file contains a type inference algorithm for a simply typed language.
% It serves as a core language on which more complex languages can
% be built through type signatures for the language constants.
% NOTE: this does not run in LambdaProlog version 2.6 due to
%	the absence of implication.
% NOTE: properly this module should be parameterized over the
%	predicate 'contype'.

module tp_simple.

import tp_types.

type  termtype  term -> term -> tp -> o.
type  vartype   term -> tp -> o.
type  contype   term -> tp -> o.      		% external.

% termtype U V Alpha
% takes a partially typed term U and returns fully typed term V.
% Alpha may be a constraint (the type we know that U should
% have) or a result (the infered type of U).

% vartype X Beta
% records the types that were infered for bound variables.
% The types of free variables should be given in the environment,
% but may of course be variable.  See tp_simple_ex for an example.

% The simplest clause here would be
% termtype (olam     U) (lam Beta V) (Beta ==> Gamma) :-
%    (pi X\ (vartype X Beta => termtype (U X) (V X) Gamma)).
% but that would be unnecessarily inefficient.

termtype (olam     U) (lam Beta V) Alpha :- !,
  Alpha = (Beta ==> Gamma),	   
  (pi X\ (vartype X Beta => termtype (U X) (V X) Gamma)).

termtype (lam Beta U) (lam Beta V) Alpha :- !,
  Alpha = (Beta ==> Gamma),	  
  (pi X\ (vartype X Beta => termtype (U X) (V X) Gamma)).

termtype (U0 @ U1) (V0 @ V1) Alpha :- !,
  termtype U0 V0 (Beta ==> Alpha),
  termtype U1 V1 Beta.

% It is tempting to include a cut in the following clause, as in
% termtype X X Alpha :- vartype X Alpha, !.
% but this would be incomplete if there is more than one
% unifier of Alpha with the saved type of X, only the first one would
% be returned.  Thus this program could not be extended to the second-order
% polymorphic lambda calculus.

termtype X X Alpha :- vartype X Alpha.

termtype C C Alpha :- contype C Alpha.
